/*
 *	Areafix features
 *	(LazyFix Project)
 *   
 *	Public Domain: may be copied and sold freely
 */

#include	<stdio.h>
#include	<stdlib.h>
#include	<time.h>
#include	<ctype.h>
#include	<string.h>

#include	"lfix.h"
#include 	"areafix.h"
#include	"match.h"
#include 	"misc.h"
#include	"config.h"

/* ---------------- process areafix */

int do_areafix(char *sysop,char *password,int zone,int net,int node,int point,int filefix)
{
	FILE *tf;
	FILE *outmsg;
	char tempfix[BBSSTR],tempfix2[BBSSTR];
	char tempstr[BBSSTR],temparea[BBSSTR];
	char areafile[BBSSTR],passwo[BBSSTR];
	int error=0,modify=0;
	
	if(filefix)
		strcpy(areafile,FAREASFILE);
	else
		strcpy(areafile,AREASFILE);
	
	/* tempfile for original message */
	strcpy(tempfix,get_ticktemp());
	strcat(tempfix,LFIXTEMP1);
	/* tempfile for reply */
	strcpy(tempfix2,get_ticktemp());
	strcat(tempfix2,LFIXTEMP2);
	
	/* open input message */
	tf=fopen(tempfix,"r");
	if(!tf)
	{
		logline(3,"Can't open tempfile %s",tempfix);
		error++;
	}
	else
	{
		/* open output file */
		outmsg=fopen(tempfix2,"w");
		if(!outmsg)
		{
			logline(3,"Cant open tempfile %s",tempfix2);
			error++;
		}
		else
		{
			/* check pw */
			get_password(passwo,zone,net,node,point);
			if( strnicmp(password,passwo,strlen(passwo)) || strlen(passwo)<2 )
			{
				if(strlen(passwo)>=2)
				{
					password[30]=0;
					logline(1,"Bad password %s/%s",password,passwo);
					fprintf(outmsg,"Bad password!\n");
				}
				else
				{
					logline(1,"Node not known to us");
					fprintf(outmsg,"You are unknown!\n");
				}
				error++;
			}
			else
			{
				if(open_areas(areafile)==BBSFAIL)
				{
					logline(1,"Can't open %s",areafile);
					error++;
				}
				else
				{
					while(fgets(tempstr,BBSSTR-2,tf)!=NULL)
					{
						if(*tempstr!=1) /* kludge */
						{
							strcln(tempstr,-1);
							if(tempstr[0]=='+')
							{
								strspacecpy(temparea,tempstr+1);
								if(subscribe(outmsg,temparea,zone,net,node,point,filefix)==BBSOK)
									modify++;
							}
							else if(tempstr[0]=='-')
							{
								strspacecpy(temparea,tempstr+1);
								if(unsubscribe(outmsg,temparea,zone,net,node,point)==BBSOK)
									modify++;
							}
							else if(tempstr[0]!='%' && tempstr[0]!=0) /* ignore all % lines */
							{
								tempstr[30]=0;
								fprintf(outmsg,"ERROR: Unknown command \"%s\"\n",tempstr);
							}
						}
					} /* endwhile */
					/* now actually change */
					if(modify)
					{
						logline(1,"Changing %s file",areafile);
						change_areas(outmsg,areafile,zone,net,node,point);
					}
					close_areas();
					
					/* display list */
					fprintf(outmsg,"\n\n------------------------------------------------------------\n");
					fprintf(outmsg,"The following area(s) are available (*=subscribed):\n\n");
				
					if(open_areas(areafile)==BBSFAIL)
					{
						logline(2,"Can't open %s!!",areafile);
							error++;
					}
					else
					{
						display_areas(outmsg,zone,net,node,point,filefix);
						close_areas();
					}
				} /* end open areas */
			} /* end password */
			fprintf(outmsg,"\n-+- %s version %s (compiled on %s)\n\n",
					LAZYNAME,LAZYVERS,__DATE__);
			fclose(outmsg);
		} /* end open outmsg */
		fclose(tf);
	} /* end open input mail */
	
	logline(1,"Writing reply to %s (%d:%d/%d.%d)",sysop,
		zone,net,node,point);
	sprintf(tempstr,"%d:%d/%d.%d",zone,net,node,point);
	postmsg( ( (filefix==0) ? AREAFIXN1 : FILEFIXN1),NULL,
			sysop,tempstr,"LazyFix reply",
			get_netmail(),tempfix2,1);

	if(remove(tempfix2))
		logline(3,"Can't remove %s",tempfix2);
		
	if(error)
		return BBSFAIL;
	return BBSOK;
}

/* ----------------- is subscriber */

FIDONODE *is_subs(FIDONODE *first,int zone,int net,int node,int point)
{
	FIDONODE *currentnode;
	
	currentnode=first;
	while(currentnode)
	{
		if(currentnode->zone==zone
				&& currentnode->net==net
				&& currentnode->node==node
				&& currentnode->point==point)
			return currentnode;
		currentnode=currentnode->next;
	}
	return NULL;
}

/* ---------------- permissions */

int is_allowed(char *group, int zone,int net,int node,int point,int filefix)
{
	BFIXPERM *curp;
	char atemp[BBSSTR];
	
	sprintf(atemp,"%d:%d/%d.%d",zone,net,node,point);
	
	if(filefix)	
		curp=get_fperms();
	else
		curp=get_perms();
		
	if(!curp)
		return BBSFAIL;
	
	if(!group || strlen(group)==0)
		return BBSOK; /* area has no group -> everyone */
		
	while(curp)
	{
		if(!stricmp(curp->group,group))
		{/* our group */
			if(match(curp->nodes,atemp)==TRUE)
				return BBSOK; /* allowed */
		}
		curp=curp->next;
	}
		
	return BBSFAIL;
}

/* ---------------- process areafix lines */

int unsubscribe(FILE *outf, char *area, int zone,int net,int node,int point)
{
	ECHOLIST *mylist;
	
	if(!outf) return BBSFAIL;

	mylist=get_area(area);
	if(!mylist)
	{
		fprintf(outf,"ERROR: Area %s doesn't exist!\n",area);
		logline(2,"Unsub. failed: area %s doesn't exist",area);
		return BBSFAIL;
	}
	if(is_subs(mylist->firstnode,zone,net,node,point)==NULL)
	{
		fprintf(outf,"ERROR: You don't receive %s!\n",area);
		logline(2,"Unsub. failed: user doesn't receives %s",area);
		return BBSFAIL;
	}

	mylist->modify=-1;
	fprintf(outf,"Area %s removed.\n",area);
	logline(1,"User unsubscribed from area %s (%s)",area,mylist->group);
	return BBSOK;
}

int subscribe(FILE *outf,char *area, int zone,int net,int node,int point,int filefix)
{
	ECHOLIST *mylist;
	
	if(!outf) return BBSFAIL;

	mylist=get_area(area);
	if(!mylist)
	{
		fprintf(outf,"ERROR: Area %s doesn't exist!\n",area);
		logline(2,"Sub. failed: Area %s doesn't exist",area);
		return BBSFAIL;
	}
	if(is_allowed(mylist->group,zone,net,node,point,filefix)==BBSFAIL)
	{
		fprintf(outf,"ERROR: Access to area %s denied.\n",area);
		logline(2,"Sub. failed: access denied to area %s (%s)",area,mylist->group);
		return BBSFAIL;
	}
	if(is_subs(mylist->firstnode,zone,net,node,point))
	{
		fprintf(outf,"ERROR: You already receive %s!\n",area);
		logline(2,"Sub. failed: User already receives %s",area);
		return BBSFAIL;
	}
	
	mylist->modify=1;
	fprintf(outf,"Area %s added.\n",area);
	logline(1,"User subscribed to area %s (%s)",area,mylist->group);
	return BBSOK;
}

/* ---------------- display/change areas */

extern ECHOLIST *el; /* global in misc.c*/
extern int maxel;

/* change areas */

int change_areas(FILE *outf, char *areafile,int zone,int net,int node,int point)
{
	char oldareafile[BBSSTR];
	char buffer[BBSSTR];
	FILE *olda,*newa;
	FIDONODE *mynode;
	FIDONODE prim;
	int error=0,curline=-1,i;
	
	/* makes old areas file */
	strcpy(oldareafile,areafile);
	if(strlen(areafile)<5)
		return BBSFAIL;
	strcpy(oldareafile+strlen(oldareafile)-3,"old");
	
	/* primary address */
	get_ouraddress(&prim,0,0,0,0);
	
	remove(oldareafile); /* remove existing backup */
	/* renames current areas to old */
	if(rename(areafile,oldareafile))
	{
		logline(3,"Can't rename %s to %s",areafile,oldareafile);
		error++;
	}
	else
	{
		/* open old file */	
		olda=fopen(oldareafile,"r");
		if(!olda)
		{
			logline(3,"Can't open %s",oldareafile);
			error++;
		}
		else
		{
			newa=fopen(areafile,"w");
			if(!newa)
			{
				logline(3,"Can't open %s",areafile);
				error++;
			}
			else
			{
				/* now read the old file */
				while(fgets(buffer,BBSSTR-2,olda)!=NULL)
				{
					curline++; /* current areas.bbs line number */
					strcln(buffer,-1);
					
					for(i=0;i<maxel;i++)
					{
						if(curline==el[i].line && el[i].modify)
						{ /* it's a modifiable line and the correct one */
							if(!strnicmp(buffer,el[i].file,strlen(el[i].file)))
							{ /* dble checked, now modify */
								if(el[i].modify==1)
								{
									if(!is_subs(el[i].firstnode,zone,net,node,point))
									{
										logline(1,"Adding %d:%d/%d.%d for %s",
											zone,net,node,point,el[i].name);
										if(strlen(buffer)<BBSSTR-15) /* room? */
										{
											if(buffer[strlen(buffer)-1]!=' ')
												strcat(buffer," ");
											
											if(point==0)
												sprintf(buffer+strlen(buffer),"%d:%d/%d",zone,net,node);
											else
											{
												if(prim.zone==zone && prim.net==net && prim.node==node)
													sprintf(buffer+strlen(buffer),".%d",point);
												else
													sprintf(buffer+strlen(buffer),"%d:%d/%d.%d",
														zone,net,node,point);
											}												
										}
										else
											logline(3,"Area line %d too long",curline);
									} /* endsub */
								} /* end add */
								else if(el[i].modify==-1)
								{
									mynode=is_subs(el[i].firstnode,zone,net,node,point);
									if(mynode)
									{
										logline(1,"Removing %d:%d/%d.%d for %s",
											zone,net,node,point,el[i].name);
											
										if(mynode->next==NULL)
											buffer[mynode->position]=0;
										else
											strcpy(buffer+(mynode->position),buffer+((mynode->next)->position));
									}
								} /* end remove */
							}
							else
								logline(3,"Double check failed (%d)",curline);
						} /* modifyiable line */
					} /* endfor */
					
					fprintf(newa,"%s\n",buffer);
				}
				fclose(newa);
			}
		
			fclose(olda);
		}
	} /*rename*/
	
	if(error)
		return BBSFAIL;
	return BBSOK;
}

/* display areafix result */

int display_areas(FILE *outf, int zone, int net, int node, int point, int filefix)
{
	int i;
	char subchar;
	
	if(!el || !outf)
		return BBSFAIL;
	
	for(i=0;i<maxel;i++)
	{
		if(is_allowed(el[i].group,zone,net,node,point,filefix)==BBSOK)
		{	
			subchar=' ';
			if(is_subs(el[i].firstnode,zone,net,node,point))
				subchar='*';
			fprintf(outf,"%c%-20.20s %s\n",subchar,el[i].name,el[i].desc);
		} /* !allowed */
	}
	
	return BBSOK;
}
